"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  useHookall: () => useHookall,
  useHookallSync: () => useHookallSync
});
module.exports = __toCommonJS(src_exports);

// src/Hookall.ts
var HookallStore = class extends WeakMap {
  ensure(obj, key) {
    if (!this.has(obj)) {
      const scope2 = {};
      this.set(obj, scope2);
    }
    const scope = this.get(obj);
    if (!Object.prototype.hasOwnProperty.call(scope, key)) {
      scope[key] = /* @__PURE__ */ new Map();
    }
    return scope[key];
  }
};
var Hookall = class _Hookall {
  static Global = {};
  static _Store = new HookallStore();
  beforeHooks;
  afterHooks;
  /**
   * Create hook system. you can pass a target object or undefined.
   * If you pass a object, the hook system will be work for object locally. You're going to want this kind of usage in general.
   * If not specified, will be work for global. This is useful when you want to share your work with multiple files.
   * @param target The object to work with locally. If not specified, will be work for global.
   */
  constructor(target) {
    this.beforeHooks = _Hookall._Store.ensure(target, "before");
    this.afterHooks = _Hookall._Store.ensure(target, "after");
  }
  _ensureCommand(hooks, command) {
    if (!hooks.has(command)) {
      hooks.set(command, []);
    }
    return hooks.get(command);
  }
  _createWrapper(command, callback, repeat) {
    return {
      callback,
      command,
      repeat
    };
  }
  _on(hooks, command, callback, repeat) {
    const wrappers = this._ensureCommand(hooks, command);
    const wrapper = this._createWrapper(command, callback, repeat);
    wrappers.unshift(wrapper);
  }
  /**
   * You register a preprocessing function, which is called before the callback function of the `trigger` method.
   * The value returned by this function is passed as a parameter to the `trigger` method's callback function.
   * If you register multiple preprocessing functions, they are executed in order, with each function receiving the value returned by the previous one as a parameter.
   * @param command Command to work.
   * @param callback Preprocessing function to register.
   */
  onBefore(command, callback) {
    this._on(this.beforeHooks, command, callback, -1);
    return this;
  }
  /**
   * Similar to the `onBefore` method, but it only runs once.
   * For more details, please refer to the `onBefore` method.
   * @param command Command to work.
   * @param callback Preprocessing function to register.
   */
  onceBefore(command, callback) {
    this._on(this.beforeHooks, command, callback, 1);
    return this;
  }
  /**
   * You register a post-processing function which is called after the callback function of the `trigger` method finishes.
   * This function receives the value returned by the `trigger` method's callback function as a parameter.
   * If you register multiple post-processing functions, they are executed in order, with each function receiving the value returned by the previous one as a parameter.
   * @param command Command to work.
   * @param callback Post-preprocessing function to register.
   */
  onAfter(command, callback) {
    this._on(this.afterHooks, command, callback, -1);
    return this;
  }
  /**
   * Similar to the `onAfter` method, but it only runs once.
   * For more details, please refer to the `onAfter` method.
   * @param command Command to work.
   * @param callback Post-preprocessing function to register.
   */
  onceAfter(command, callback) {
    this._on(this.afterHooks, command, callback, 1);
    return this;
  }
  _off(hooks, command, callback) {
    const wrappers = this._ensureCommand(hooks, command);
    if (callback) {
      const i = wrappers.findIndex((wrapper) => wrapper.callback === callback);
      if (i !== -1) {
        wrappers.splice(i, 1);
      }
    } else {
      wrappers.length = 0;
    }
    return this;
  }
  /**
   * You remove the preprocessing functions registered with `onBefore` or `onceBefore` methods.
   * If you don't specify a callback parameter, it removes all preprocessing functions registered for that command.
   * @param command Commands with preprocessing functions to be deleted.
   * @param callback Preprocessing function to be deleted.
   */
  offBefore(command, callback) {
    this._off(this.beforeHooks, command, callback);
    return this;
  }
  /**
   * You remove the post-preprocessing functions registered with `onAfter` or `onceAfter` methods.
   * If you don't specify a callback parameter, it removes all post-preprocessing functions registered for that command.
   * @param command Commands with post-preprocessing functions to be deleted.
   * @param callback post-Preprocessing function to be deleted.
   */
  offAfter(command, callback) {
    this._off(this.afterHooks, command, callback);
    return this;
  }
  async _hookWith(hooks, command, value) {
    let wrappers = this._ensureCommand(hooks, command);
    let i = wrappers.length;
    while (i--) {
      const wrapper = wrappers[i];
      value = await wrapper.callback(value);
      wrapper.repeat -= 1;
      if (wrapper.repeat === 0) {
        this._off(hooks, command, wrapper.callback);
      }
    }
    return value;
  }
  /**
   * You execute the callback function provided as a parameter. This callback function receives the 'initialValue' parameter.
   * 
   * If preprocessing functions are registered, they run first, and the value returned by the preprocessing functions becomes the 'initialValue' parameter.
   * After the callback function finishes, post-processing functions are called.
   * These post-processing functions receive the value returned by the callback function as a parameter and run sequentially.
   * 
   * The final value returned becomes the result of the `trigger` method.
   * @param command Command to work.
   * @param initialValue Initial value to be passed to the callback function.
   * @param callback The callback function to be executed.
   */
  async trigger(command, initialValue, callback) {
    let value;
    value = await this._hookWith(this.beforeHooks, command, initialValue);
    value = await callback(value);
    value = await this._hookWith(this.afterHooks, command, value);
    return value;
  }
};
function useHookall(target = Hookall.Global) {
  return new Hookall(target);
}

// src/HookallSync.ts
var HookallStore2 = class extends WeakMap {
  ensure(obj, key) {
    if (!this.has(obj)) {
      const scope2 = {};
      this.set(obj, scope2);
    }
    const scope = this.get(obj);
    if (!Object.prototype.hasOwnProperty.call(scope, key)) {
      scope[key] = /* @__PURE__ */ new Map();
    }
    return scope[key];
  }
};
var HookallSync = class _HookallSync {
  static Global = {};
  static _Store = new HookallStore2();
  beforeHooks;
  afterHooks;
  /**
   * Create hook system. you can pass a target object or undefined.
   * If you pass a object, the hook system will be work for object locally. You're going to want this kind of usage in general.
   * If not specified, will be work for global. This is useful when you want to share your work with multiple files.
   * @param target The object to work with locally. If not specified, will be work for global.
   */
  constructor(target) {
    this.beforeHooks = _HookallSync._Store.ensure(target, "before");
    this.afterHooks = _HookallSync._Store.ensure(target, "after");
  }
  _ensureCommand(hooks, command) {
    if (!hooks.has(command)) {
      hooks.set(command, []);
    }
    return hooks.get(command);
  }
  _createWrapper(command, callback, repeat) {
    return {
      callback,
      command,
      repeat
    };
  }
  _on(hooks, command, callback, repeat) {
    const wrappers = this._ensureCommand(hooks, command);
    const wrapper = this._createWrapper(command, callback, repeat);
    wrappers.unshift(wrapper);
  }
  /**
   * You register a preprocessing function, which is called before the callback function of the `trigger` method.
   * The value returned by this function is passed as a parameter to the `trigger` method's callback function.
   * If you register multiple preprocessing functions, they are executed in order, with each function receiving the value returned by the previous one as a parameter.
   * @param command Command to work.
   * @param callback Preprocessing function to register.
   */
  onBefore(command, callback) {
    this._on(this.beforeHooks, command, callback, -1);
    return this;
  }
  /**
   * Similar to the `onBefore` method, but it only runs once.
   * For more details, please refer to the `onBefore` method.
   * @param command Command to work.
   * @param callback Preprocessing function to register.
   */
  onceBefore(command, callback) {
    this._on(this.beforeHooks, command, callback, 1);
    return this;
  }
  /**
   * You register a post-processing function which is called after the callback function of the `trigger` method finishes.
   * This function receives the value returned by the `trigger` method's callback function as a parameter.
   * If you register multiple post-processing functions, they are executed in order, with each function receiving the value returned by the previous one as a parameter.
   * @param command Command to work.
   * @param callback Post-preprocessing function to register.
   */
  onAfter(command, callback) {
    this._on(this.afterHooks, command, callback, -1);
    return this;
  }
  /**
   * Similar to the `onAfter` method, but it only runs once.
   * For more details, please refer to the `onAfter` method.
   * @param command Command to work.
   * @param callback Post-preprocessing function to register.
   */
  onceAfter(command, callback) {
    this._on(this.afterHooks, command, callback, 1);
    return this;
  }
  _off(hooks, command, callback) {
    const wrappers = this._ensureCommand(hooks, command);
    if (callback) {
      const i = wrappers.findIndex((wrapper) => wrapper.callback === callback);
      if (i !== -1) {
        wrappers.splice(i, 1);
      }
    } else {
      wrappers.length = 0;
    }
    return this;
  }
  /**
   * You remove the preprocessing functions registered with `onBefore` or `onceBefore` methods.
   * If you don't specify a callback parameter, it removes all preprocessing functions registered for that command.
   * @param command Commands with preprocessing functions to be deleted.
   * @param callback Preprocessing function to be deleted.
   */
  offBefore(command, callback) {
    this._off(this.beforeHooks, command, callback);
    return this;
  }
  /**
   * You remove the post-preprocessing functions registered with `onAfter` or `onceAfter` methods.
   * If you don't specify a callback parameter, it removes all post-preprocessing functions registered for that command.
   * @param command Commands with post-preprocessing functions to be deleted.
   * @param callback post-Preprocessing function to be deleted.
   */
  offAfter(command, callback) {
    this._off(this.afterHooks, command, callback);
    return this;
  }
  _hookWith(hooks, command, value) {
    let wrappers = this._ensureCommand(hooks, command);
    let i = wrappers.length;
    while (i--) {
      const wrapper = wrappers[i];
      value = wrapper.callback(value);
      wrapper.repeat -= 1;
      if (wrapper.repeat === 0) {
        this._off(hooks, command, wrapper.callback);
      }
    }
    return value;
  }
  /**
   * You execute the callback function provided as a parameter. This callback function receives the 'initialValue' parameter.
   * 
   * If preprocessing functions are registered, they run first, and the value returned by the preprocessing functions becomes the 'initialValue' parameter.
   * After the callback function finishes, post-processing functions are called.
   * These post-processing functions receive the value returned by the callback function as a parameter and run sequentially.
   * 
   * The final value returned becomes the result of the `trigger` method.
   * @param command Command to work.
   * @param initialValue Initial value to be passed to the callback function.
   * @param callback The callback function to be executed.
   */
  trigger(command, initialValue, callback) {
    let value;
    value = this._hookWith(this.beforeHooks, command, initialValue);
    value = callback(value);
    value = this._hookWith(this.afterHooks, command, value);
    return value;
  }
};
function useHookallSync(target = HookallSync.Global) {
  return new HookallSync(target);
}
